@@ -1,25 +1,10 @@
 /*
-    Copyright 2012  Andrius da Costa Ribas <andriusmao@gmail.com>
+    SPDX-FileCopyrightText: 2012 Andrius da Costa Ribas <andriusmao@gmail.com>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) version 3, or any
-    later version accepted by the membership of KDE e.V. (or its
-    successor approved by the membership of KDE e.V.), which shall
-    act as a proxy defined in Section 6 of version 3 of the license.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
 
-#ifndef LIBKGAPI2_DRIVEPERMISSION_H
-#define LIBKGAPI2_DRIVEPERMISSION_H
+#pragma once
 
 #include "object.h"
 #include "types.h"
@@ -49,12 +34,15 @@ class KGAPIDRIVE_EXPORT Permission: public KGAPI2::Object
 {
 
   public:
+
     enum Role {
         UndefinedRole = -1,
         OwnerRole = 0,
         ReaderRole = 1,
         WriterRole = 2,
         CommenterRole = 3,
+        OrganizerRole = 4,
+        FileOrganizerRole = 5,
     };
 
     enum Type {
@@ -65,6 +53,65 @@ class KGAPIDRIVE_EXPORT Permission: public KGAPI2::Object
         TypeAnyone = 3
     };
 
+    /**
+     * @brief Details of whether the permissions on this shared drive item are
+     * inherited or directly on this item. This is an output-only field which
+     * is present only for shared drive items.
+     */
+    class PermissionDetails
+    {
+
+      public:
+        enum PermissionType {
+            UndefinedType = -1,
+            TypeFile = 0,
+            TypeMember = 1,
+        };
+
+        PermissionDetails();
+        PermissionDetails(const PermissionDetails &other);
+        ~PermissionDetails();
+        bool operator==(const PermissionDetails &other) const;
+        bool operator!=(const PermissionDetails &other) const { return !operator==(other); }
+
+        /**
+         * @brief The permission type for this user.
+         */
+        PermissionDetails::PermissionType permissionType() const;
+
+        /**
+         * @brief The primary role for this user.
+         */
+        Permission::Role role() const;
+
+        /**
+         * @brief Additional roles for this user. Only commenter is currently possible,
+         * though more may be supported in the future.
+         */
+        QList<Permission::Role> additionalRoles() const;
+
+        /**
+         * @brief The ID of the item from which this permission is inherited.
+         * This is an output-only field and is only populated for members of
+         * the shared drive.
+         */
+        QString inheritedFrom() const;
+
+        /**
+         * @brief Whether this permission is inherited. This field is always populated. This is an output-only field.
+         */
+        bool inherited() const;
+
+      private:
+        class Private;
+        QScopedPointer<Private> const d;
+        friend class Private;
+        friend class Permission;
+    };
+
+    using PermissionDetailsPtr = QSharedPointer<PermissionDetails>;
+    using PermissionDetailsList = QList<PermissionDetailsPtr>;
+
     explicit Permission();
     explicit Permission(const Permission &other);
     ~Permission() override;
@@ -169,6 +216,37 @@ class KGAPIDRIVE_EXPORT Permission: public KGAPI2::Object
      */
     void setValue(const QString &value);
 
+    /**
+     * @brief The email address of the user or group this permission refers to.
+     * This is an output-only field which is present when the permission type is
+     * user or group.
+     */
+    QString emailAddress() const;
+
+    /**
+     * @brief The domain name of the entity this permission refers to.
+     * This is an output-only field which is present when the permission
+     * type is user, group or domain.
+     */
+    QString domain() const;
+
+    /**
+     * @brief The time at which this permission will expire.
+     */
+    QDateTime expirationDate() const;
+
+    /**
+     * @brief Whether the account associated with this permission has been
+     * deleted. This field only pertains to user and group permissions.
+     */
+    bool deleted() const;
+
+    /**
+     * @brief Details of whether the permissions on this shared drive
+     * item are inherited or directly on this item.
+     */
+    Permission::PermissionDetailsList permissionDetails() const;
+
     static PermissionPtr fromJSON(const QByteArray &jsonData);
     static PermissionsList fromJSONFeed(const QByteArray &jsonData);
     static QByteArray toJSON(const PermissionPtr &permission);
@@ -184,4 +262,3 @@ class KGAPIDRIVE_EXPORT Permission: public KGAPI2::Object
 
 } /* namespace KGAPI2 */
 
-#endif // LIBKGAPI2_DRIVEPERMISSION_H
